/*
 * main.c
 *
 *  Created on: 19 lip 2015
 *      Author: Szymon
 */

#include <avr/interrupt.h>
#include <avr/io.h>
//#include <avr/iotn13.h>
#include <avr/sleep.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

#define PWM_DOOR_PIN 7
#define PWM_LAMP_PIN 2
#define REFLECTORS_PIN 1
#define LIGHT_PIN 0
#define UNLOCK_PIN 3
#define LOCK_PIN 2
#define DOOR_PIN 1
#define ENGINE_PIN 0

static inline uint8_t pin_get(uint8_t number) {
	if (PINA & (1 << number)) {
		return 0;
	}
	return 1;
}

static inline void pin_set(uint8_t number, uint8_t value) {
	if (value) {
		PORTB |= 1 << number;
	} else {
		PORTB &= ~(1 << number);
	}
}

static inline uint8_t pin_event(uint8_t number, uint8_t rise) {
	static uint8_t last = 0;
	uint8_t current = pin_get(number) << number;
	uint8_t ret_val = 0;
	if (!rise) {
		if (current > (last & (1 << number))) {
			ret_val = 1;
		}
	} else {
		if (current < (last & (1 << number))) {
			ret_val = 1;
		}
	}
	if (current) {
		last |= current;
	} else {
		last &= ~(1 << number);
	}
	return ret_val;
}

static inline void dim_lamp_update(uint8_t up) {
	static uint8_t value = 0;
	if (up) {
		if (value < 255) {
			value++;
		}
	} else {
		if (value) {
			value--;
		}
	}
	OCR0A = 255 - value;
}

static inline void dim_door_update(uint8_t up) {
	static uint8_t value = 0;
	if (up) {
		if (value < 255) {
			value++;
		}
	} else {
		if (value) {
			value--;
		}
	}
	OCR0B = 255 - value;
}

ISR(TIM0_OVF_vect) {
	static uint16_t door = 0;
	static uint16_t lamp = 0;
	static uint16_t light = 0;
	static uint16_t engine = 0;
	if (!pin_get(ENGINE_PIN)) {
		if (pin_get(DOOR_PIN)) {
			door = 2;
			lamp = 2;
		} else {
			door = 0;
			lamp = 0;
		}
		light = 0;
	} else {
		if (pin_event(DOOR_PIN, 0)) {
			door = 8000;
			lamp = 8000;
			light = 0;
		}
		if (pin_event(UNLOCK_PIN, 0)) {
			door = 4000;
			light = 1000;
		}
		if (pin_event(DOOR_PIN, 1)) {
			door = 1500;
			lamp = 1500;
			light = 0;
		}
		if (pin_event(LOCK_PIN, 0)) {
			door = 0;
			lamp = 0;
			light = 1000;
		}
		engine = 500;
	}
	if (door) {
		door--;
	}
	if (lamp) {
		lamp--;
	}
	if (light) {
		light--;
	}
	if (engine) {
		engine--;
	}
	dim_door_update(door ? 1 : 0);
	dim_lamp_update(lamp ? 1 : 0);
	pin_set(LIGHT_PIN, light ? 1 : 0);
	pin_set(REFLECTORS_PIN, engine ? 0 : 1);
}

int main(void) {
	DDRA = 1 << PWM_DOOR_PIN;
	DDRB = (1 << LIGHT_PIN) | (1 << PWM_LAMP_PIN) | (1 << REFLECTORS_PIN);
	PORTA = 0xFF & (~(1 << PWM_DOOR_PIN));
	PORTB = 0xFF & (~((1 << LIGHT_PIN) | (1 << PWM_LAMP_PIN) | (1 << REFLECTORS_PIN)));

	OCR0A = 255;
	OCR0B = 255;
	TCCR0A = (1 << COM0A1) |(1 << COM0A0) | (1 << COM0B1) | (1 << COM0B0) | (1 << WGM01) | (1 << WGM00);
	TCCR0B = (1 << CS01) | (1 << CS00);

	TIMSK0 |= 1 << TOIE0;

	set_sleep_mode(SLEEP_MODE_IDLE);
	sei();
	while (1) {
        sleep_enable();
        sleep_cpu();
        sleep_disable();
	}
	return 0;
}
